local AddonName, Hermes = ...

local SenderStatusTimer = nil
local SENDER_STATUS_UPDATE_FREQUENCY = 1

function Hermes:RemoveAllSenders()
	while #Hermes.Senders > 0 do
		self:RemoveSender(Hermes.Senders[1])
	end
end

function Hermes:FindSenderByName(name)
	for _, sender in ipairs(Hermes.Senders) do
		if(name == sender.name) then
			return sender
		end
	end
	
	return nil
end

function Hermes:AddSender(name, class)
	sender = {
		name = name,
		class = class,
		zone = nil,
		online = true, --assume online
		dead = false, --assume alive
	}
	
	tinsert(Hermes.Senders, sender)
	
	--update sender but do not allow events to be sent
	self:UpdateSenderStatus(sender, nil)
	
	--now call the event
	Hermes:FireEvent("OnSenderAdded", sender)
end

function Hermes:RemoveSender(sender)
	--first remove all instances of the sender in abilities
	self:RemoveAbilityInstancesForSender(sender)
	--self:UpdateAbilityAvailabilityForAll()
	
	--call the event
	Hermes:FireEvent("OnSenderRemoved", sender)
	
	--now delete from table
	Hermes:DeleteFromIndexedTable(Hermes.Senders, sender)
end

function Hermes:StopSenderStatusTimer()
	if(SenderStatusTimer) then
		self:CancelTimer(SenderStatusTimer, true)
		SenderStatusTimer = self:ScheduleTimer("OnSenderStatusTimer", 0.1, nil) --convert to one shot timer
		self:CancelTimer(SenderStatusTimer, true) --cancel again, we don't care if it ran or not
		SenderStatusTimer = nil
	end
end

function Hermes:StartSenderStatusTimer()
	SenderStatusTimer = self:ScheduleRepeatingTimer("OnSenderStatusTimer", SENDER_STATUS_UPDATE_FREQUENCY, nil)
end

function Hermes:OnSenderStatusTimer()
	if(Hermes.Senders) then
		for _, sender in ipairs(Hermes.Senders) do
			self:UpdateSenderStatus(sender, true)
		end
	end
end

--performance/memory optimization to define them once and continue using them below
local _wasAvailable = nil
local _isAvailable = nil
local _oldZone = nil
local _oldOnline = nil
local _oldDead = nil
	
function Hermes:UpdateSenderStatus(sender, allowEvents)

	if allowEvents then
		--remember sender state for events later
		_oldZone = sender.zone
		_oldOnline = sender.online
		_oldDead = sender.dead
		_wasAvailable = self:IsSenderAvailable(sender)
	end
	
	--update sender properties
	if Hermes.Player.raid then
		local raidId = UnitInRaid(sender.name)
		if raidId then
			local name, _, _, _, _, _, zone, online, dead, _, _ = GetRaidRosterInfo(raidId)
			
			if(name == sender.name) then
				sender.zone = zone
				sender.online = online == 1
				sender.dead = dead == 1
			else
				error("Unexpected sender status during UpdateSenderStatus")
			end
		else
			--sender dropped from raid
			Hermes:RemoveSender(sender)
			return --don't send any more events
		end
		
	elseif Hermes.Player.party then
		if UnitInParty(sender.name) then
			sender.zone = Hermes.Player.zone --checking zones doesn't work in a party, so just assume it matches player
			--sender.online = true
			sender.online = UnitIsConnected(sender.name) == 1
			
			sender.dead = UnitIsDead(sender.name) == 1 or UnitIsGhost(sender.name) == 1
		else
			--sender dropped from party
			Hermes:RemoveSender(sender)
			return --don't send any more events
		end
	else
		--config mode
		sender.zone = Hermes.Player.zone --checking zones doesn't work in a party, so just assume it matches player
		sender.online = true
		sender.dead = UnitIsDead(sender.name) == 1 or UnitIsGhost(sender.name) == 1
	end
	
	
	--see if sender is available again
	_isAvailable = self:IsSenderAvailable(sender)
		
	--update zone changes, but don't update if yourself
	if allowEvents and _oldZone ~= sender.zone and Hermes.Player.name ~= sender.name then
		--print("UpdateSenderStatus zone changed: "..sender.name)
		Hermes:FireEvent("OnSenderZoneChanged", sender)
	end
		
	--update online changes	
	if allowEvents and _oldOnline ~= sender.online then
		Hermes:FireEvent("OnSenderOnlineChanged", sender)
	end
		
	--update dead changes
	if allowEvents and _oldDead ~= sender.dead then
		Hermes:FireEvent("OnSenderDeadChanged", sender)
	end
		
	--update availability changes
	if _wasAvailable ~= _isAvailable then
		--fire event about the sender status changing
		if allowEvents then
			Hermes:FireEvent("OnSenderAvailabilityChanged", sender)
		end
			
		--now fire events for any ability and ability instances being tracked that belong to this sender
		if allowEvents then
			for _, instance in ipairs(Hermes.AbilityInstances) do
				if(instance.sender == sender) then
					--fire event on instance
					if allowEvents then
						Hermes:FireEvent("OnAbilityAvailableSendersChanged", instance.ability)
						Hermes:FireEvent("OnAbilityInstanceAvailabilityChanged", instance)
					end
				end
			end
		end
	end
end

function Hermes:IsSenderAvailable(sender)
	--don't count zone if sender is also player
	if Hermes.Player.name ~= sender.name then
		return Hermes.Player.zone == sender.zone and sender.online == true and sender.dead == false
	else
		return sender.online == true and sender.dead == false
	end
end